package org.communiquons.comunic.independentnotifications;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Build;
import android.os.IBinder;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.NotificationCompat;
import androidx.core.content.ContextCompat;

import org.communiquons.comunic.MainActivity;
import org.communiquons.comunic.R;

public class NotificationsService extends Service {
    public static final String CHANNEL_ID = "IndependentPushServiceChannel";
    private static final String INDEPENDENT_PUSH_NOTIFICATIONS_SERVICE = "independent-push-notifications-service";
    private static final String WS_URL_PREF_KEY = "ws_url";

    public static void configure(@NonNull String wsURL, @NonNull Context context) {
        SharedPreferences prefs = context.getSharedPreferences(INDEPENDENT_PUSH_NOTIFICATIONS_SERVICE, MODE_PRIVATE);

        if (prefs.getString(WS_URL_PREF_KEY, "").equals(wsURL))
            return;

        stopService(context);
        prefs.edit().putString(WS_URL_PREF_KEY, wsURL).apply();
        startService(context);
    }

    public static void disable(@NonNull Context context) {
        SharedPreferences prefs = context.getSharedPreferences(INDEPENDENT_PUSH_NOTIFICATIONS_SERVICE, MODE_PRIVATE);
        prefs.edit().remove(WS_URL_PREF_KEY).apply();

        stopService(context);
    }

    public static void stopService(@NonNull Context context) {
        Intent serviceIntent = new Intent(context, NotificationsService.class);
        context.stopService(serviceIntent);
    }

    public static void startService(@NonNull Context context) {
        SharedPreferences prefs = context.getSharedPreferences(INDEPENDENT_PUSH_NOTIFICATIONS_SERVICE, MODE_PRIVATE);

        if (!prefs.contains(WS_URL_PREF_KEY)) {
            System.err.println("Independent push notifications service not configured ! Skipping!");
            return;
        }

        Intent intent = new Intent(context, NotificationsService.class);
        ContextCompat.startForegroundService(context, intent);
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        createNotificationChannel();

        Intent notificationIntent = new Intent(this, MainActivity.class);
        PendingIntent pendingIntent = PendingIntent.getActivity(this,
                0, notificationIntent, 0);
        Notification notification = new NotificationCompat.Builder(this, CHANNEL_ID)
                .setContentTitle("Comunic")
                .setContentText(getText(R.string.independent_push_notification_notification_text))
                .setSmallIcon(R.drawable.splash_icon)
                .setContentIntent(pendingIntent)
                .build();
        startForeground(1, notification);
        
        initService();
        
        return START_STICKY;
    }

    private void createNotificationChannel() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            NotificationChannel serviceChannel = new NotificationChannel(
                    CHANNEL_ID,
                    "Foreground Service Channel",
                    NotificationManager.IMPORTANCE_DEFAULT
            );
            NotificationManager manager = getSystemService(NotificationManager.class);
            manager.createNotificationChannel(serviceChannel);
        }
    }

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }
    
    private void initService() {
        String url = getSharedPreferences(INDEPENDENT_PUSH_NOTIFICATIONS_SERVICE, MODE_PRIVATE)
                .getString(WS_URL_PREF_KEY, null);
        
        System.out.println("START HEAVY WORK HERE !!!!");
        System.out.println("Connect to " + url);
    }
}
